import unittest

from adapt.intent import IntentBuilder
from heare.config import SettingsDefinition

from heare.client.events.message import Message
from heare.skills import Skill
from util import run_in_container


class SkillBindingTest(unittest.TestCase):
    def test_on_handler(self):
        fooed = {}

        def callback(message: Message):
            fooed[message.metadata.get('class')] = True

        class MySkill(Skill):
            def load(self, settings: SettingsDefinition):
                pass

            def on_foo(self, intent_message: Message):
                self.speak('oh hi!', metadata={'class': MySkill.__name__})

        container, client = run_in_container([], MySkill)
        client.on('speak', callback)
        client.run_forever()

        client.emit(Message(message_type='foo', data={}))
        self.assertTrue(
            fooed.get(MySkill.__name__),
            f"fooed[{MySkill.__name__}] should be set to True"
        )


    def test_decorator(self):
        fooed = {}

        def callback(message: Message):
            fooed[message.metadata.get('class')] = True

        class MyDecoratedSkill(Skill):
            def load(self, settings: SettingsDefinition):
                pass

            @Skill.handles_intent(IntentBuilder('foo').build())
            def foo(self, intent_message: Message):
                self.speak('oh hi!', metadata={'class': MyDecoratedSkill.__name__})

        container, client = run_in_container([], MyDecoratedSkill)
        client.on('speak', callback)
        client.run_forever()

        client.emit(Message(message_type='foo', data={}))
        self.assertTrue(
            fooed.get(MyDecoratedSkill.__name__),
            f"fooed[{MyDecoratedSkill.__name__}] should be set to True"
        )
