from collections import defaultdict
from typing import Type, Tuple, List

from heare.client.events.message import Message
from heare.logging import get_logger
from heare.skills.container import HeareContainer

logger = get_logger('heare.test.util')


class TestEventSource(object):
    def __init__(self):
        self.callbacks = defaultdict(list)
        self.one_time_callbacks = defaultdict(list)

    def on(self, event_name, handler):
        self.callbacks[event_name].append(handler)

    def once(self, event_name, handler):
        self.one_time_callbacks[event_name].append(handler)

    def emit(self, message: Message):
        for callback in self.callbacks[message.message_type]:
            try:
                callback(message)
            except:
                logger.exception(f"Error invoking {callback} for message {message}.")

        for callback in self.one_time_callbacks[message.message_type]:
            try:
                callback(message)
            except:
                logger.exception(f"Error invoking {callback} for message {message}.")

        self.one_time_callbacks[message.message_type].clear()

        for callback in self.callbacks['*']: # magic wildcard!
            try:
                callback(message)
            except:
                logger.exception(
                    f"Error invoking wildcard {callback} for message {message}.")

    def run_forever(self, ping_interval=0):
        for callback in self.callbacks.get('open', []):
            try:
                callback()
            except BaseException as e:
                logger.exception(f"Error invoking wildcard {callback} on open.")

        for callback in self.one_time_callbacks.get('open'):
            try:
                callback()
            except:
                logger.exception(f"Error invoking {callback} on open.")

        self.one_time_callbacks.get('open', []).clear()


def run_in_container(args: List[str], *skill_type: Type) \
        -> Tuple[HeareContainer, TestEventSource]:
    client = TestEventSource()
    container = HeareContainer(skill_type, client, args)
    container.bootstrap()
    return container, client

